/***** BEGIN LICENSE BLOCK *****
   - Version: MPL 1.1/GPL 2.0/LGPL 2.1
   -
   - The contents of this file are subject to the Mozilla Public License Version
   - 1.1 (the "License"); you may not use this file except in compliance with
   - the License. You may obtain a copy of the License at
   - http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS IS" basis,
   - WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
   - for the specific language governing rights and limitations under the
   - License.
   -
   - The Original Code is "FlashGot".
   -
   - The Initial Developer of the Original Code is Giorgio Maone.
   - Portions created by the Initial Developer are Copyright (C) 2004
   - the Initial Developer. All Rights Reserved.
   -
   - Contributor(s): Giorgio Maone <g.maone @ informaction.com>
   -
   - Alternatively, the contents of this file may be used under the terms of
   - either the GNU General Public License Version 2 or later (the "GPL"), or
   - the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
   - in which case the provisions of the GPL or the LGPL are applicable instead
   - of those above. If you wish to allow use of your version of this file only
   - under the terms of either the GPL or the LGPL, and not to allow others to
   - use your version of this file under the terms of the MPL, indicate your
   - decision by deleting the provisions above and replace them with the notice
   - and other provisions required by the LGPL or the GPL. If you do not delete
   - the provisions above, a recipient may use your version of this file under
   - the terms of any one of the MPL, the GPL or the LGPL.
   -
   - ***** END LICENSE BLOCK *****/

#include "stdafx.h"

#define FLASHGOT_FLASHGET 1
#import "progid:JetCar.Netscape.1"
using namespace JCCATCHLib;


#define FLASHGOT_NETTRANSPORT 2
#import "progid:NTIEHelper.NTIEAddUrl.1"
using namespace NTIEHELPERLib;


#define BUF_SIZE 1024

char g_buf[BUF_SIZE];
wchar_t g_bufw[BUF_SIZE];

#define str2bstr_t(s) _bstr_t (MultiByteToWideChar(CP_UTF8,0,s,-1,g_bufw,(int)sizeof(g_bufw)) ? g_bufw : L"" ) 	


char* readLine(FILE *stream) 
{
	char *res=fgets(g_buf,BUF_SIZE,stream);
	if(res) 
	{
		size_t lastPos=strlen(res)-1;
		while(lastPos>=0 && 
			(res[lastPos]==0x0a || res[lastPos]==0x0d)
			) res[lastPos--]='\0';
	}
	return res;
}




void fail(char *msg, int code) {
	MessageBox(NULL, msg,
					  "FlashGot error",
					  MB_OK | MB_ICONERROR);
	exit(code);
}

void checkComponents() {
	int retVal=0;
	CoInitialize(NULL);
	try {
		IJetCarNetscapePtr jcns(__uuidof(JetCarNetscape));
	} catch(...) {
		retVal|=FLASHGOT_FLASHGET;
	}
	try {
		INTIEAddUrlPtr ntie(__uuidof(NTIEAddUrl));
	} catch(...) {
		retVal|=FLASHGOT_NETTRANSPORT;
	}
	exit(retVal);
}



int main(int argc, char* argv[])
{
	char *fname;
	FILE *f;
	
	
	if(argc<2)
	{
		checkComponents();
	} 
	else if(!(f=fopen(fname=argv[1],"rb")))  
	{
		sprintf(g_buf,"Can't open file %s",fname);
		fail(g_buf,0x01000);
	}
	else
	{	
		if(!feof(f))	
		{
		
			CoInitialize(NULL); 
			try 
			{
				int linksCount=atoi(readLine(f));
				int lineCount=linksCount*2+1;

				char *strDM=strchr(g_buf,';');
				int dm=strDM && (strlen(strDM)>1) ? atoi(++strDM) : FLASHGOT_FLASHGET;
				
				bstr_t *parms =new bstr_t[lineCount];
				
				int narr=dm==FLASHGOT_NETTRANSPORT?2:1;
				
				SAFEARRAY *psa[2];
				int elemCount=narr==1?lineCount:linksCount;

				for(int a=narr; a-->0;) {
					SAFEARRAYBOUND bounds [1]; 
					bounds[0].cElements=elemCount;
					bounds[0].lLbound=0;
					psa[a]=SafeArrayCreate(VT_VARIANT, 1, bounds);
				}

				VARIANT v;
				v.vt=VT_BSTR;
				long ix[]={0};

				switch(dm) {
					case FLASHGOT_NETTRANSPORT:
						{ 
							parms[0] = str2bstr_t(readLine(f));
							for(int j=1; ix[0]<elemCount; ix[0]++) {
								for(int a=2; a-->0;) {
									v.bstrVal=(parms[j++]=str2bstr_t(readLine(f))).GetBSTR();
									SafeArrayPutElement(psa[a], ix, &v);
								}
							}

							VARIANT *urls, *remarks;
							SafeArrayAccessData(psa[1], (void **)&urls);
							SafeArrayAccessData(psa[0], (void **)&remarks);
							INTIEAddUrlPtr ntie(__uuidof(NTIEAddUrl));
							ntie->AddList(parms[0],urls, remarks);
							SafeArrayUnlock(psa[1]);
							SafeArrayUnlock(psa[0]);
						}
						break;
					default:
						{
							for (; ix[0] < elemCount; ix[0]++) {
								v.bstrVal=(parms[ix[0]]=str2bstr_t(readLine(f))).GetBSTR();
								SafeArrayPutElement(psa[0], ix, &v);
							}

							v.vt = VT_ARRAY | VT_VARIANT | VT_BYREF; // VBScript Array
							v.pparray=&psa[0];
							IJetCarNetscapePtr jcns(__uuidof(JetCarNetscape));
							jcns->AddUrlList(&v);
						}
				}

				
				
			} catch(...) {
				fail("Download manager not properly installed.",0x02000);
			}
			CoUninitialize();
		}
		
		else 
		{
			sprintf(g_buf,"Temporary file %s is empty",fname);
			fail(g_buf,0x03000);
		}
		fclose(f);
		remove(fname);
	}
	
}

int WINAPI
WinMain(HINSTANCE inst, HINSTANCE previnst, LPSTR cmdline, int cmdshow)
{
    return main(__argc, __argv); 
}
