/***** BEGIN LICENSE BLOCK *****
   - Version: MPL 1.1/GPL 2.0/LGPL 2.1
   -
   - The contents of this file are subject to the Mozilla Public License Version
   - 1.1 (the "License"); you may not use this file except in compliance with
   - the License. You may obtain a copy of the License at
   - http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS IS" basis,
   - WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
   - for the specific language governing rights and limitations under the
   - License.
   -
   - The Original Code is "FlashGot".
   -
   - The Initial Developer of the Original Code is Giorgio Maone.
   - Portions created by the Initial Developer are Copyright (C) 2004
   - the Initial Developer. All Rights Reserved.
   -
   - Contributor(s): Giorgio Maone <g.maone @ informaction.com>
   -
   - Alternatively, the contents of this file may be used under the terms of
   - either the GNU General Public License Version 2 or later (the "GPL"), or
   - the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
   - in which case the provisions of the GPL or the LGPL are applicable instead
   - of those above. If you wish to allow use of your version of this file only
   - under the terms of either the GPL or the LGPL, and not to allow others to
   - use your version of this file under the terms of the MPL, indicate your
   - decision by deleting the provisions above and replace them with the notice
   - and other provisions required by the LGPL or the GPL. If you do not delete
   - the provisions above, a recipient may use your version of this file under
   - the terms of any one of the MPL, the GPL or the LGPL.
   -
   - ***** END LICENSE BLOCK *****/


function FlashGot() {
  if(!this.inited) {
    
    const shell=Components.classes['@mozilla.org/appshell/appShellService;1'
      ].getService(Components.interfaces.nsIAppShellService);
    
    var globals=shell.hiddenDOMWindow._flashGot_globals;
    if(typeof(globals)!="object") {
      globals=shell.hiddenDOMWindow._flashGot_globals=new FlashGotGlobals(); 
    }
    for(var prop in globals) {
      this[prop]=globals[prop]; 
    }
    
    this.inited=true; 
  }
}

FlashGot.prototype.switchOption=function(opt) {
  this.setPref(opt,!this.getPref(opt));
}

FlashGot.prototype.switchMenus=function() {
  const invalidLink=!this.isValidLink(this.getCurrentLink());
  const selText=this.getWindow().getSelection().toString();
  const invalidSel=! (selText.length>0 && /[^\s]+/.test(selText));
  const noLink=this.getLinksCount()==0 ;
  for(var j=FLASHGOT_DMS.length; j-->0;) {
    var dm=FLASHGOT_DMS[j];
    var unsupported=this.unsupported & dm;
    var hideDisabled=this.getPref("hideDisabledCmds");
    
    function menuSwitch(name, disabled) {
     var menuItem=document.getElementById("flashgot-menuitem-"+name+dm);
     if(menuItem && ! ( menuItem.hidden = unsupported ) ) {
       menuItem.setAttribute("disabled",disabled?"true":"false");
       menuItem.hidden=disabled && hideDisabled;
     }
    }
    
    menuSwitch("it",  invalidLink);
    menuSwitch("sel", invalidSel);
    menuSwitch("all", noLink);
  
  }
  
  const opts=document.getElementById("flashgot-menu-options"
    ).getElementsByTagName("menuitem");
  for(var j=opts.length; j-->0;) {
    var menuItem=opts[j];
    var match=menuItem.id?menuItem.id.match(/opt-(.*)/):null;
    if(match) {
      menuItem.setAttribute("checked",
        this.getPref(match[1])?"true":"false"
        );
    }
  }
}

FlashGot.prototype.menuShowing=function(ev) {
  this.switchMenus();
}

FlashGot.prototype.getWindow=function() {
  return document.commandDispatcher.focusedWindow;
}

FlashGot.prototype.getDocument=function() {
  return this.getWindow().document;
}

FlashGot.prototype.getCurrentLink=function() {
  var elem = document.popupNode;
  while(elem) {
    if(elem instanceof Components.interfaces.nsIDOMHTMLAnchorElement && elem.href)
      return elem;
    elem = elem.parentNode;
  }
  return null;
}

FlashGot.prototype.getLinksCount=function() {
  const doc=this.getDocument();
  var count=doc.links.length;
  if(this.getPref("includeImages")) count+=doc.images.length;
  return count;
}

FlashGot.prototype.getLinks=function(filter) {
  var allLinks=filterLinks(this.getDocument().links,filter);
  
  if(this.getPref("includeImages")) {
    allLinks=allLinks.concat(filterLinks(
      this.getDocument().images,
      function(l) {
        l.href=l.src
        l.title=l.alt+" ("+l.width+","+l.height+")";
        return filter(l);
      }));
  }
  return allLinks;
  
  function filterLinks(allLinks,filter) {
    const links=new Array();
    for(var j=0,len=allLinks.length; j<len; j++) {
      var l=allLinks[j];
      if(filter(l)) {
        links[links.length]=l;
      }
    }
    return links;
  }
}

FlashGot.prototype.getReferrer=function() {
  return this.getDocument().location.href;
}

FlashGot.prototype.parseDesc=function(link) {
  return (typeof(link.title)=="string" && link.title.length>0
    ?link.title:link.innerHTML).replace(
      /<.*?>/g,"").replace(/\s+/g," ");
}


FlashGot.prototype.isValidLink=function(link) {
  // return /^(http|https|ftp|mms|rtsp):\/\/.*/i.test(link.href);
  // regex above is too limitating, now that we support NetTransport too
  return link && link.href && /^[a-z]+:\/\/.*/i.test(link.href);
}

FlashGot.prototype.downloadLink=function() {
  const link=this.getCurrentLink();
  if(link != null && this.isValidLink(link)) {
    this.download([link], arguments.length==0?FLASHGOT_FLASHGET:arguments[0]);
  }
}


FlashGot.prototype.downloadSel=function() {
  this.selection=this.getWindow().getSelection();
  var links=this.getLinks(function(link) {
    return flashGot.selection.containsNode(link,true) && flashGot.isValidLink(link)
  });
 
  const httpFixer=/^h.{2}p/i;
  const noprotFixer=/^([a-z]+[a-z\d]+\.[a-z\d\.]+\/)/i;
  const m=this.selection.toString().match(/\b(h.{2}p:\/\/|ftp:\/\/|mms:\/\/|rtsp:\/\/|[a-z]+[a-z\d]+\.[a-z\d\.]+\/)[^\s]*/gi);
  if(m) {
    var textLinks=new Array();
    for(var j=0, len=m.length; j<len; j++) {
      var href=m[j].replace(httpFixer,"http").replace(noprotFixer,"http://$1");
      for(var k=links.length; k-->0;) {
        if(links[k].href.toLowerCase()==href.toLowerCase()) {
          href=null; break;
        }
      }
      if(href) {
        textLinks[textLinks.length]={href: href, title: m[j], innerHTML: "", fg_fake: true};
      }
    }
    links=links.concat(textLinks);
  }
  
  this.download(links,arguments.length==0?FLASHGOT_FLASHGET:arguments[0]);
}

FlashGot.prototype.downloadAll=function() {
  const links=this.getLinks(function(link) {
    return flashGot.isValidLink(link) 
  });
  this.download(links,arguments.length==0?FLASHGOT_FLASHGET:arguments[0]);
}

FlashGot.prototype.download=function(links,manager) {
  if(links.length==0) return;

  var job=(links.length)+";"+manager+"\n" 
          + this.getReferrer();
  for(var j=0, len=links.length; j<len; j++) {
    job+="\n"+links[j].href+"\n"+this.parseDesc(links[j]);
  }
  this.performJob(job);
}


FlashGot.prototype.performJob=function(job) {
  const cc=Components.classes;
  const ci=Components.interfaces;
  const unicodeConverter = cc["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(
      ci.nsIScriptableUnicodeConverter);
  unicodeConverter.charset = "UTF-8";
  job=unicodeConverter.ConvertFromUnicode(job);
  
  const jobFile=cc["@mozilla.org/file/local;1"].createInstance(ci.nsILocalFile);
  jobFile.initWithPath(this.tmpDir.path);
  jobFile.append("flashgot.fgt");
  jobFile.createUnique(0,-1);
  const os=cc["@mozilla.org/network/file-output-stream;1"].createInstance(
    ci.nsIFileOutputStream);
  os.init(jobFile,0x02,-1,0);
  os.write(job,job.length);
  os.close();
  
  this.runNative([jobFile.path],false);
  
}




function FlashGotGlobals() {
  const cc=Components.classes;
  const ci=Components.interfaces;
  
  this.fileLocator=cc["@mozilla.org/file/directory_service;1"].getService(
    ci.nsIProperties);
  this.tmpDir=this.fileLocator.get("TmpD",ci.nsIFile);
  
  this.prefService=cc["@mozilla.org/preferences-service;1"].getService(
    ci.nsIPrefService);
  this.prefs=this.prefService.getBranch("flashgot.");
  this.setupLegacyPrefs();
  
  
  this.exeFile=cc["@mozilla.org/file/local;1"].createInstance(ci.nsILocalFile);
  this.exeFile.initWithPath(this.tmpDir.path);
  this.exeFile.append("FlashGot.exe");
  this.createExecutable();
  
  this.unsupported=this.runNative([],true);
  this.inited=true;
}

FlashGotGlobals.prototype.setupLegacyPrefs=function() {
  try {
  var file=this.fileLocator.get("ProfD",Components.interfaces.nsIFile);
  file.append("pref");
  file.append("flashgot.js");
  if(!file.exists()) return;
    this.prefService.readUserPrefs(file);
  } catch(e) {
    window.dump(e.message);
  }
}

FlashGotGlobals.prototype.createExecutable=function() {
  const cc=Components.classes;
  const ci=Components.interfaces;
  const ios=cc['@mozilla.org/network/io-service;1'].getService(ci.nsIIOService);
  const bis=cc['@mozilla.org/binaryinputstream;1'].createInstance(ci.nsIBinaryInputStream);
  var channel;
  bis.setInputStream((channel=
    ios.newChannel("chrome://flashgot/content/FlashGot.exe",null,null)).open());
  const bytesCount=channel.contentLength;
  
  const os=cc["@mozilla.org/network/file-output-stream;1"].createInstance(
    ci.nsIFileOutputStream);
  
  try {
    os.init(this.exeFile,0x02 | 0x08, -1, 0);
    var bos=cc['@mozilla.org/binaryoutputstream;1'].createInstance(ci.nsIBinaryOutputStream);
    bos.setOutputStream(os);
    bos.writeByteArray(bis.readByteArray(bytesCount),bytesCount);
    bos.close();
  } catch(ioerr) { // locked?
    if(this.exeFile.exists()) { // security check: it must be the right exe!
      const testBis=cc['@mozilla.org/binaryinputstream;1'].createInstance(
        ci.nsIBinaryInputStream);
      testBis.setInputStream(
        (channel=ios.newChannelFromURI(ios.newFileURI(this.exeFile))).open());
      const error=new Error("Old, corrupt or unlegitemately modified "
        +this.exeFile.path
        +".\nThe file is locked: please delete it manually\n");
        +ioerr.message;
      if(channel.contentLength!=bytesCount) throw error;
     
      const legitimateData=bis.readByteArray(bytesCount);
      const testData=testBis.readByteArray(bytesCount);
      for(var j=bytesCount; j-->0;) {
        if(legitimateData[j]!=testData[j]) throw new error;
      }
    } else throw ioerr;
  } finally {
    os.close();
  }
  bis.close();
  return true;
}

FlashGotGlobals.prototype.runNative=function(args,blocking) {
  if(this.exeFile.exists() || this.createExecutable()) {
    const proc=Components.classes['@mozilla.org/process/util;1'].createInstance(
      Components.interfaces.nsIProcess);
    proc.init(this.exeFile);
    proc.run(blocking,args,args.length,{});
    return proc.exitValue;
  } else {
    return 0xff;
  }
}

FlashGotGlobals.prototype.getPref=function(name,def) {
  const IPC=Components.interfaces.nsIPrefBranch;
  try {
    switch(this.prefs.getPrefType(name)) {
      case IPC.PREF_STRING:
        return this.prefs.getCharValue(name);
      case IPC.PREF_INT:
        return this.prefs.getIntPref(name);
      case IPC.PREF_BOOL:
        return this.prefs.getBoolPref(name);
    }
  } catch(e) {}
  return def;
}

FlashGotGlobals.prototype.setPref=function(name,value) {
  switch(typeof(value)) {
    case "string":
        this.prefs.setCharValue(name,value);
        break;
    case "boolean":
      this.prefs.setBoolPref(name,value);
      break;
    case "number":
      this.prefs.setIntPref(name,value);
      break;
    default:
      throw new Error("Unsupported type "+typeof(value)+" for preference "+name);
  }
}

const FLASHGOT_FLASHGET=1;
const FLASHGOT_NETTRANSPORT=2;

const FLASHGOT_DMS=[ FLASHGOT_FLASHGET, FLASHGOT_NETTRANSPORT ];

window.addEventListener("load",  
    function() {
      flashGot=new FlashGot();
      document.getElementById("contentAreaContextMenu"
        ).addEventListener("popupshowing", function(ev) {
          flashGot.menuShowing(ev)
        }, false);
    }, false);
